@@ -75,23 +75,23 @@ def create_db():
 	db_path = get_path_db()
 	db_exists = os.path.exists(db_path)
 	if db_exists:
-		print("\n=> Skipping database file creation as a database file already exists at path:\n" + db_path + "\n")
+		print(f"\n=> Skipping database file creation as a database file already exists at path:\n{db_path}\n")
 	else:
 		# Create sqlite file for db.
 		try:
 			db = get_db()
 		except:
-			print("\n=> Error - failed to create database file at path:\n" + db_path)
+			print(f"\n=> Error - failed to create database file at path:\n{db_path}")
 			print("===================================\n")
 			raise
-		print("\n=> Success - created database file for machine learning metrics at path:\n" + db_path + "\n")
+		print(f"\n=> Success - created database file for machine learning metrics at path:\n{db_path}\n")
 
 	db = get_db()
 	# Create tables inside db.
 	tables = db.get_tables()
 	table_count = len(tables)
 	if table_count > 0:
-		print("\n=> Info - skipping table creation as the following tables already exist:\n" + str(tables) + "\n")
+		print(f"\n=> Info - skipping table creation as the following tables already exist:\n{tables}\n")
 	else:
 		db.create_tables([
 			Dataset, Label, Featureset, 
@@ -102,7 +102,7 @@ def create_db():
 		tables = db.get_tables()
 		table_count = len(tables)
 		if table_count > 0:
-			print("\n=> Success - created the following tables within database:\n" + str(tables) + "\n")
+			print(f"\n=> Success - created the following tables within database:\n{tables}\n")
 		else:
 			print("\n=> Error - failed to create tables. Please see README file section titled: 'Deleting & Recreating the Database'\n")
 
@@ -115,13 +115,13 @@ def delete_db(confirm:bool=False):
 			try:
 				os.remove(db_path)
 			except:
-				print("\n=> Error - failed to delete database file at path:\n" + db_path)
+				print(f"\n=> Error - failed to delete database file at path:\n{db_path}")
 				print("===================================")
 				raise
-			print("\n=> Success - deleted database file at path:\n" + db_path + "\n")
+			print(f"\n=> Success - deleted database file at path:\n{db_path}\n")
 
 		else:
-			print("\n=> Info - there is no file to delete at path:\n" + db_path + "\n")
+			print(f"\n=> Info - there is no file to delete at path:\n{db_path}\n")
 	else:
 		print("\n=> Info - skipping deletion because `confirm` arg not set to boolean `True`.\n")
 
@@ -210,7 +210,7 @@ class Dataset(BaseModel):
 		, rename_columns:list = None
 	):
 		if dataframe.empty:
-			raise ValueError("\nYikes - The dataframe you provided is empty according to `df.empty`")
+			raise ValueError("\nYikes - The dataframe you provided is empty according to `df.empty`\n")
 
 		Dataset.check_file_format(file_format)
 		if rename_columns is not None:
@@ -289,7 +289,7 @@ class Dataset(BaseModel):
 				# generate string-based column names to feed to pandas
 				col_count = ndarray.shape[1]
 				column_names = [str(i) for i in range(col_count)]
-				print("\nInfo - You didn't provide any column names for your array, so we generated them for you.\ncolumn_names: " + str(column_names) + "\n" )
+				print(f"\nInfo - You didn't provide any column names for your array, so we generated them for you.\ncolumn_names: {column_names}\n")
 			
 		shape = {}
 		shape['rows'], shape['columns'] = ndarray.shape[0], ndarray.shape[1]
@@ -429,14 +429,14 @@ class Dataset(BaseModel):
 	def check_file_format(file_format):
 		accepted_formats = ['csv', 'tsv', 'parquet', None]
 		if file_format not in accepted_formats:
-			raise ValueError("\nYikes - Available file formats include uncompressed csv, tsv, and parquet.\nYour file format: " + file_format + "\n")
+			raise ValueError(f"\nYikes - Available file formats include uncompressed csv, tsv, and parquet.\nYour file format: {file_format}\n")
 
 
 	def check_column_count(user_columns, structure):
 		col_count = len(user_columns)
 		structure_col_count = structure.shape[1]
 		if col_count != structure_col_count:
-			raise ValueError("\nYikes - The dataframe you provided has " + structure_col_count + "columns, but you provided " + col_count + "columns.\n")
+			raise ValueError(f"\nYikes - The dataframe you provided has <{structure_col_count}> columns, but you provided <{col_count}> columns.\n")
 
 
 	def compress_or_not(data, perform_gzip):
@@ -480,7 +480,7 @@ class Label(BaseModel):
 				l_cols = l.columns
 				l_cols_alpha = sorted(l_cols)
 				if cols_aplha == l_cols_alpha:
-					raise ValueError("\nYikes - This Dataset already has Label <id:" + l_id + "> with the same columns.\nCannot create duplicate.\n")
+					raise ValueError(f"\nYikes - This Dataset already has Label <id:{l_id}> with the same columns.\nCannot create duplicate.\n")
 
 		column_count = len(columns)
 
@@ -587,7 +587,7 @@ class Featureset(BaseModel):
 				else:
 					f_cols_alpha = None
 				if cols_aplha == f_cols_alpha:
-					raise ValueError("\nYikes - This Dataset already has Featureset <id:" + f_id + "> with the same columns.\nCannot create duplicate.\n")
+					raise ValueError(f"\nYikes - This Dataset already has Featureset <id:{f_id}> with the same columns.\nCannot create duplicate.\n")
 
 		f = Featureset.create(
 			dataset = d
@@ -858,6 +858,21 @@ class Splitset(BaseModel):
 		return foldset
 
 
+	def make_preprocess(
+		id:int
+		, description:str = None
+		, encoder_features:object = None
+		, encoder_labels:object = None
+	):
+		preprocess = Preprocess.from_splitset(
+			splitset_id = id
+			, description = description
+			, encoder_features = encoder_features
+			, encoder_labels = encoder_labels
+		)
+		return preprocess
+
+
 
 
 class Foldset(BaseModel):
@@ -897,7 +912,7 @@ class Foldset(BaseModel):
 		train_count = len(arr_train_indices)
 		remainder = train_count % fold_count
 		if remainder != 0:
-			print("\nAdvice - The length <" + str(train_count) + "> of your training Split is not evenly divisible by the number of folds <" + str(fold_count) + "> you specified.\nThere's a chance that this could lead to misleadingly low accuracy for the last Fold, which only has <" + str(remainder) + "> samples in it.\n")
+			print(f"\nAdvice - The length <{train_count}> of your training Split is not evenly divisible by the number of folds <{fold_count}> you specified.\nThere's a chance that this could lead to misleadingly low accuracy for the last Fold, which only has <{remainder}> samples in it.\n")
 
 		foldset = Foldset.create(
 			fold_count = fold_count
@@ -931,7 +946,7 @@ class Foldset(BaseModel):
 
 		if fold_index is not None:
 			if (0 > fold_index) or (fold_index > fold_count):
-				raise ValueError("\nYikes - This Foldset <id:" + str(id) +  "> has fold indices between 0 and " + str(fold_count - 1) + "\n")
+				raise ValueError(f"\nYikes - This Foldset <id:{id}> has fold indices between 0 and {fold_count-1}\n")
 
 		s = foldset.splitset
 		supervision = s.supervision
@@ -948,7 +963,7 @@ class Foldset(BaseModel):
 		for i in fold_frames.keys():
 			
 			fold = folds[i]
-			# here, `.keys()` are 'folds_train_combined' and 'fold_validation'
+			# At the next level down, `.keys()` are 'folds_train_combined' and 'fold_validation'
 			for fold_name in fold.samples.keys():
 
 				# placeholder for the frames/arrays
@@ -987,8 +1002,9 @@ class Fold(BaseModel):
 	- The `samples` attribute contains the indices of `folds_train_combined` and `fold_validation`, 
 	  where `fold_validation` is the rotating fold that gets left out.
 	"""
-	fold_index = IntegerField()
+	fold_index = IntegerField() # order within the Foldset.
 	samples = JSONField()
+	# contains_all_classes = BooleanField()
 	
 	foldset = ForeignKeyField(Foldset, backref='folds')
 
@@ -1059,6 +1075,36 @@ class Algorithm(BaseModel):
 	description = CharField(null=True)
 
 
+	def make_hyperparamset(
+		id:int
+		, hyperparameters:dict
+		, preprocess_id:int = None
+		, description:str = None
+	):
+		hyperparamset = Hyperparamset.from_algorithm(
+			algorithm_id = id
+			, hyperparameters = hyperparameters
+			, preprocess_id = preprocess_id
+			, description = description
+		)
+		return hyperparamset
+
+
+	def make_batch(
+		id:int
+		, splitset_id:int
+		, hyperparamset_id:int = None
+		, foldset_id:int = None
+	):
+
+		batch = Batch.from_algorithm(
+			algorithm_id = id
+			, splitset_id = splitset_id
+			, hyperparamset_id = hyperparamset_id
+			, foldset_id = foldset_id
+		)
+		return batch
+
 
 
 class Hyperparamset(BaseModel):
@@ -1160,27 +1206,29 @@ class Batch(BaseModel):
 		, splitset_id:int
 		, hyperparamset_id:int = None
 		, foldset_id:int = None
-		, only_folded_training = False # inclusion of splitset.samples["train"]
 	):
 		algorithm = Algorithm.get_by_id(algorithm_id)
 		splitset = Splitset.get_by_id(splitset_id)
 
-		folds = [None]
+		
 		if foldset_id is not None:
 			foldset =  Foldset.get_by_id(foldset_id)
 			foldset_splitset = foldset.splitset
 			if foldset_splitset != splitset:
-				raise ValueError("\nYikes - The Foldset <id:" + str(foldset_id) + "> and Splitset <id:" + str(splitset_id) + "> you provided are not related.\n")
-			folds = folds + list(foldset.folds)
-			if only_folded_training is True:
-				folds.remove(None)
+				raise ValueError(f"\nYikes - The Foldset <id:{foldset_id}> and Splitset <id:{splitset_id}> you provided are not related.\n")
+			folds = list(foldset.folds)
+		else:
+			# Just so we have an item to loop over as a null condition when creating Jobs.
+			folds = [None]
 
 		if hyperparamset_id is not None:
 			hyperparamset = Hyperparamset.get_by_id(hyperparamset_id)
 			combos = list(hyperparamset.hyperparamcombos)
 		else:
+			# Just so we have an item to loop over as a null condition when creating Jobs.
 			combos = [None]
 
+		# Here `[None]` just multiplies by 1.
 		job_count = len(combos) * len(folds)
 
 		b = Batch.create(
@@ -1188,10 +1236,10 @@ class Batch(BaseModel):
 			, job_count = job_count
 			, algorithm = algorithm
 			, splitset = splitset
+			, foldset = foldset
 			, hyperparamset = hyperparamset
 		)
 
-		# Both of these lists (folds, combos) already handle null scenarios.
 		for f in folds:
 			for c in combos:
 				Job.create(
@@ -1228,7 +1276,7 @@ class Batch(BaseModel):
 		proc_name = "aidb_batch_" + str(batch.id)
 		proc_names = [p.name for p in multiprocessing.active_children()]
 		if proc_name in proc_names:
-			raise ValueError("\nYikes - Cannot start this Batch because multiprocessing.Process.name '" + proc_name + "' is already running.\n")
+			raise ValueError(f"\nYikes - Cannot start this Batch because multiprocessing.Process.name '{proc_name}' is already running.\n")
 
 		statuses = Batch.get_statuses(id)
 		all_not_started = (set(statuses.values()) == {'Not yet started'})
@@ -1261,7 +1309,7 @@ class Batch(BaseModel):
 		proc_name = "aidb_batch_" + str(batch.id)
 		proc_names = [p.name for p in multiprocessing.active_children()]
 		if proc_name not in proc_names:
-			raise ValueError("\nYikes - Cannot terminate `multiprocessing.Process.name` '" + proc_name + "' because it is not running.\n")
+			raise ValueError(f"\nYikes - Cannot terminate `multiprocessing.Process.name` '{proc_name}' because it is not running.\n")
 
 		processes = multiprocessing.active_children()
 		for p in processes:
@@ -1269,9 +1317,9 @@ class Batch(BaseModel):
 				try:
 					p.terminate()
 				except:
-					raise Exception("\nYikes - Failed to terminate `multiprocessing.Process` '" + proc_name + ".'\n")
+					raise Exception(f"\nYikes - Failed to terminate `multiprocessing.Process` '{proc_name}.'\n")
 				else:
-					print("\nKilled `multiprocessing.Process` '" + proc_name + "' spawned from Batch <id:" + str(batch.id) + ">.\n")
+					print(f"\nKilled `multiprocessing.Process` '{proc_name}' spawned from Batch <id:{batch.id}\n")
 
 
 	def metrics_to_pandas(id:int):
@@ -1393,7 +1441,7 @@ class Job(BaseModel):
 			labels_processed = np.argmax(labels_processed, axis=1)
 
 		split_metrics['accuracy'] = accuracy_score(labels_processed, predictions)
-		split_metrics['precision'] = precision_score(labels_processed, predictions, average=average)
+		split_metrics['precision'] = precision_score(labels_processed, predictions, average=average, zero_division=0)
 		split_metrics['recall'] = recall_score(labels_processed, predictions, average=average)
 		split_metrics['f1'] = f1_score(labels_processed, predictions, average=average)
 		return split_metrics
@@ -1441,11 +1489,11 @@ class Job(BaseModel):
 		j = Job.get_by_id(id)
 		if (j.status == "Succeeded"):
 			if verbose:
-				print("\nSkipping Job #" + str(j.id) + " as is has already succeeded.")
+				print(f"\nSkipping <Job.id{j.id}> as is has already succeeded.\n")
 			return j
 		elif (j.status == "Running"):
 			if verbose:
-				print("\nSkipping Job #" + str(j.id) + " as it is already running.")
+				print(f"\nSkipping <Job.id{j.id}> as it is already running.\n")
 			return j
 		else:
 			if verbose:
@@ -1457,9 +1505,12 @@ class Job(BaseModel):
 			hyperparamcombo = j.hyperparamcombo
 			fold = j.fold
 
+			"""
 			# 1. Figure out which splits the model needs to be trained and predicted against. 
-			# The `key_*` variables dynamically determine which splits to use during model_training.
-			# ^ It is being intentionally overwritten as more complex validations/ training splits are introduced.
+			- Unlike a batch, each job can have a different fold.
+			- The `key_*` variables dynamically determine which splits to use during model_training.
+			  It is being intentionally overwritten as more complex validations/ training splits are introduced.
+			"""
 			samples = {}
 			if splitset.supervision == "unsupervised":
 				samples['train'] = splitset.to_numpy(splits=['train'])['train']
@@ -1475,7 +1526,8 @@ class Job(BaseModel):
 					
 				if fold is not None:
 					foldset = fold.foldset
-					fold_samples_np = foldset.to_numpy(fold_index=fold.fold_index)[0]
+					fold_index = fold.fold_index
+					fold_samples_np = foldset.to_numpy(fold_index=fold_index)[fold_index]
 					samples['folds_train_combined'] = fold_samples_np['folds_train_combined']
 					samples['fold_validation'] = fold_samples_np['fold_validation']
 					
@@ -1485,6 +1537,7 @@ class Job(BaseModel):
 					samples['train'] = splitset.to_numpy(splits=['train'])['train']
 					key_train = "train"
 
+
 			# 2. Preprocess the features and labels.
 			# Preprocessing happens prior to training the model.
 			if preprocess is not None:
@@ -1710,7 +1763,7 @@ class Result(BaseModel):
 		a = r.job.batch.algorithm
 		analysis_type = a.analysis_type
 		if analysis_type == "regression":
-			raise ValueError("\nYikes - `Algorith.analysis_type` of 'regression' does not support this chart.\n")
+			raise ValueError("\nYikes - <Algorith.analysis_type> of 'regression' does not support this chart.\n")
 		
 
 		cm_by_split = {}
@@ -1757,7 +1810,7 @@ class Result(BaseModel):
 		a = r.job.batch.algorithm
 		analysis_type = a.analysis_type
 		if analysis_type == "regression":
-			raise ValueError("\nYikes - `Algorith.analysis_type` of 'regression' does not support this chart.\n")
+			raise ValueError("\nYikes - <Algorith.analysis_type> of 'regression' does not support this chart.\n")
 
 		pr_by_split = {}
 		for split, data in result_plot_data.items():
@@ -1817,7 +1870,7 @@ class Result(BaseModel):
 		a = r.job.batch.algorithm
 		analysis_type = a.analysis_type
 		if analysis_type == "regression":
-			raise ValueError("\nYikes - `Algorith.analysis_type` of 'regression' does not support this chart.\n")
+			raise ValueError("\nYikes - <Algorith.analysis_type> of 'regression' does not support this chart.\n")
 
 		roc_by_split = {}
 		for split, data in result_plot_data.items():
@@ -1886,6 +1939,8 @@ class Result(BaseModel):
 		fig.update_yaxes(zeroline=False, gridcolor='#262B2F', tickfont=dict(color='#818487'))
 		fig.show()
 
+def get_df():
+
 
 """
 class Environment(BaseModel)?
